@@ -1,4 +1,8 @@
 import subprocess, os, time, sys, tempfile
+if sys.version_info.major == 2:
+  import response_file
+else:
+  from tools import response_file
 
 EM_PROFILE_TOOLCHAIN = int(os.getenv('EM_PROFILE_TOOLCHAIN')) if os.getenv('EM_PROFILE_TOOLCHAIN') != None else 0
 
@@ -69,7 +73,7 @@ if EM_PROFILE_TOOLCHAIN:
     def kill(self):
       return self.process.kill()
 
-  sys.exit = profiled_sys_exit
+  exit = sys.exit = profiled_sys_exit
   subprocess.call = profiled_call
   subprocess.check_call = profiled_check_call
   subprocess.check_output = profiled_check_output
@@ -80,6 +84,13 @@ if EM_PROFILE_TOOLCHAIN:
     imaginary_pid_ = 0
     profiler_logs_path = None # Log file not opened yet
 
+    block_stack = []
+
+    # Because process spawns are tracked from multiple entry points, it is possible that record_process_start() and/or record_process_exit()
+    # are called multiple times. Prevent recording multiple entries to the logs to keep them clean.
+    process_start_recorded = False
+    process_exit_recorded = False
+
     @staticmethod
     def timestamp():
       return '{0:.3f}'.format(time.time())
@@ -99,7 +110,6 @@ if EM_PROFILE_TOOLCHAIN:
 
     @staticmethod
     def record_process_start(write_log_entry=True):
-      ToolchainProfiler.block_stack = []
       # For subprocessing.Pool.map() child processes, this points to the PID of the parent process that spawned
       # the subprocesses. This makes the subprocesses look as if the parent had called the functions.
       ToolchainProfiler.mypid_str = str(os.getpid())
@@ -108,22 +118,33 @@ if EM_PROFILE_TOOLCHAIN:
         os.makedirs(ToolchainProfiler.profiler_logs_path)
       except:
         pass
+
+      if ToolchainProfiler.process_start_recorded: return
+      ToolchainProfiler.process_start_recorded = True
+      ToolchainProfiler.block_stack = []
+
       if write_log_entry:
         with ToolchainProfiler.log_access() as f:
-          f.write('[\n')
-          f.write('{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"start","time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(sys.argv).replace('\\', '\\\\') + '"]}')
+          f.write('[\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"start","time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(sys.argv).replace('\\', '\\\\') + '"]}')
 
     @staticmethod
     def record_process_exit(returncode):
+      if ToolchainProfiler.process_exit_recorded: return
+      ToolchainProfiler.process_exit_recorded = True
+
       ToolchainProfiler.exit_all_blocks()
       with ToolchainProfiler.log_access() as f:
-        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"exit","time":' + ToolchainProfiler.timestamp() + ',"returncode":' + str(returncode) + '}')
-        f.write('\n]\n')
+        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"exit","time":' + ToolchainProfiler.timestamp() + ',"returncode":' + str(returncode) + '}\n]\n')
 
     @staticmethod
     def record_subprocess_spawn(process_pid, process_cmdline):
+      response_cmdline = []
+      for item in process_cmdline:
+        if item.startswith('@'):
+          response_cmdline += response_file.read_response_file(item)
+
       with ToolchainProfiler.log_access() as f:
-        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(process_cmdline).replace('\\', '\\\\') + '"]}')
+        f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(process_cmdline + response_cmdline).replace('\\', '\\\\') + '"]}')
 
     @staticmethod
     def record_subprocess_wait(process_pid):
@@ -135,8 +156,6 @@ if EM_PROFILE_TOOLCHAIN:
       with ToolchainProfiler.log_access() as f:
         f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"finish","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"returncode":' + str(returncode) + '}')
 
-    block_stack = []
-
     @staticmethod
     def enter_block(block_name):
       with ToolchainProfiler.log_access() as f:
@@ -183,6 +202,8 @@ if EM_PROFILE_TOOLCHAIN:
       return ToolchainProfiler.imaginary_pid_
 
 else:
+  exit = sys.exit
+
   class ToolchainProfiler:
     @staticmethod
     def record_process_start():
